YssTargets = CreateFrame("frame")
local YssTargets = YssTargets
YssTargets:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)
function YssTargets:Print(...) ChatFrame1:AddMessage(string.join(" ", "|cFF33FF99YssTargets|r:", ...)) end
YssTargets:RegisterEvent("ADDON_LOADED")

local LSM = LibStub("LibSharedMedia-3.0")
local broker = LibStub("LibDataBroker-1.1")
local icon = LibStub("LibDBIcon-1.0")
local Media = LibStub("LibSharedMedia-3.0")

Media:Register("statusbar", "Outline", [[Interface\Addons\YssTargets\Outline]])

local anchors = {
	['TOP'] = 'BOTTOM',
	['BOTTOM'] = 'TOP',
}
local sortedbars = {} -- store our active bars here
local updatetip = false -- true if the tooltip needs updating
local currentbar -- store the bar we are hovering over here

local GUIDs = {}
local sortedGUIDs = {}
local Raid = {}
local raidIDs = {}
for i=1,40 do
	raidIDs['raid'..i] = true
end
local tsort =  table.sort
local mabs = math.abs

local RAID_CLASS_COLORS = RAID_CLASS_COLORS

function YssTargets:ADDON_LOADED(event, addon)
	if addon ~= "YssTargets" then return end	


	local defaults = {
		["profile"] = {
			['width'] = 125,
			['height'] = 24,
			['point'] = 'CENTER',
			['relativePoint'] = 'CENTER',
			['xOffset'] = 0,
			['yOffset'] = 0,
			['anchor'] = 'TOP',
			['texture'] = 'Blizzard',
			['threattexture'] = 'Outline',
			['threattexturealpha'] = .75,
			['namefont'] = 'Friz Quadrata TT',
			['namefontsize'] = 11,
			['countfont'] = 'Friz Quadrata TT',
			['countfontsize'] = 11,
			['countfontr'] = 1,
			['countfontg'] = 1,
			['countfontb'] = 1,
			['countfonta'] = 1,
			['healthfont'] = 'Friz Quadrata TT',
			['healthfontsize'] = 11,
			['healthfontr'] = 1,
			['healthfontg'] = 1,
			['healthfontb'] = 1,
			['healthfonta'] = 1,
			['update'] = .2,
			['sort'] = 'normal', --or reverse
			['enemyOnly'] = nil, --tristate true if only NPC's nil if NPC only during combat and false if show everything
			['enable'] = true,
			['ToT'] = true,
			['RaidInstanceOnly'] = true,
			['LibDBIcon'] = {
				['hide'] = true, 
			},
		},
	}
	self.db = LibStub("AceDB-3.0"):New("YssTargets_DB", defaults, "Default")
	
	-- setup our drag anchor frame
	self.anchor = CreateFrame("Button")
	self.anchor:SetWidth(self.db.profile.width)
	self.anchor:SetHeight(self.db.profile.height)
	self.anchor:SetParent(UIParent)
	self.anchor:SetPoint(self.db.profile.point, UIParent, self.db.profile.relativePoint ,self.db.profile.xOffset, self.db.profile.yOffset)
	self.anchor:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 4, right = 4, top = 4, bottom = 4 }});
	self.anchor:SetBackdropBorderColor(0,1,0,.5)
	self.anchor:SetBackdropColor(0,1,0,.5)
	self.anchor:SetMovable()
	self.anchor:RegisterForDrag("LeftButton")
	self.anchor:SetScript("OnDragStart", self.anchor.StartMoving)
	self.anchor:SetScript("OnDragStop", function()
		self.db.profile.point, _, self.db.profile.relativePoint, self.db.profile.xOffset, self.db.profile.yOffset = self.anchor:GetPoint()
		self.anchor:StopMovingOrSizing()
	end)
	local text = self.anchor:CreateFontString(nil, 'OVERLAY')
		text:SetPoint("CENTER", self.anchor)
		text:SetFontObject(GameFontGreenSmall)
		text:SetJustifyH("CENTER")
		text:SetJustifyV("MIDDLE")
		text:SetText('YssTargets Anchor')
	self.anchor.text = text
	self.anchor:Hide()
	
	self.obj = LibStub("LibDataBroker-1.1"):NewDataObject("YssTargets", {
		icon = "Interface\\AddOns\\YssTargets\\icon",
		label = "|cFF33FF99YssTargets|r",
		type = "launcher",
		text  = "YssTargets",
		OnTooltipShow = function(tt)
			tt:AddLine('YssTargets')
			tt:AddLine("|cffffff00Click|r to Enable/Diable YssTargets")
			tt:AddLine("|cffffff00Middle-Click|r to toggle YssTargets Anchor")
			tt:AddLine("|cffffff00Right-Click|r to open the options menu")
		end,
		OnClick = function(clickedFrame, button)
			if button == "RightButton" then 
				InterfaceOptionsFrame_OpenToCategory(YssTargets.optframe.YssTargets)
			elseif button == "MiddleButton" then
				if YssTargets.anchor:IsVisible() then
					YssTargets.anchor:Hide()
				else
					YssTargets.anchor:Show()
				end
			else
				self.db.profile.enable = not self.db.profile.enable
				self:UpdateLDB()
				if self.db.profile.enable then
					YssTargets:Print('Enabled')
				else
					YssTargets:Print('Disabled')
				end
				wipe(GUIDs)
				wipe(sortedGUIDs)
				wipe(Raid)
				YssTargets:ResetBars()
			end
		end,
	})
	icon:Register("YssTargets", self.obj, self.db.profile.LibDBIcon)
	self.db.RegisterCallback(self, "OnProfileChanged", "ResetBars")
	self.db.RegisterCallback(self, "OnProfileCopied", "ResetBars")
	self.db.RegisterCallback(self, "OnProfileReset", "ResetBars")
	
	self:UpdateLDB()
	
	self:UnregisterEvent("ADDON_LOADED")
	self.ADDON_LOADED = nil
end

local sortf = {
	['normal'] = function(a,b)
		if GUIDs[a].count > GUIDs[b].count then
			return true
		elseif GUIDs[a].count == GUIDs[b].count and a > b then
			return true
		else
			return false
		end
	end,
	['reverse'] = function(a,b)
		if GUIDs[a].count < GUIDs[b].count then
			return true
		elseif GUIDs[a].count == GUIDs[b].count and a < b then
			return true
		else
			return false
		end
	end,
}

function YssTargets:formatNumber(n)
	if n >= 1e7 then
		return ('%.1fm'):format(n/1e6):gsub('%.?0([km])$', '%1')
	elseif n >= 1e6 then
		return ('%.2fm'):format(n/1e6):gsub('%.?0+([km])$', '%1')
	elseif n >= 1e5 then
		return ('%.0fk'):format(n/1e3)
	elseif (n >= 1e3) or (n <= -1e3) then
		return ('%.1fk'):format(n/1e3):gsub('%.?0([km])$', '%1')
	else
		return n
	end
end

local tL = 0
local updatebars = false
local function OnUpdate(self, a1)
	if tL > self.db.profile.update then
		tL = 0
		for i, bar in pairs(sortedbars) do
			if UnitExists(bar.unit) then
				bar:UpdateValue(UnitHealth(bar.unit), UnitHealthMax(bar.unit))
				if self.db.profile.ToT then
					self:UpdateToT(bar)
				elseif bar.target then
					bar.target = self:RecycleBar(bar.target)
				end
			else
				updatebars = true
			end
		end
		if updatebars then
			updatebars = false
			self:UpdateBars()
		end
	else
		tL = tL + a1
	end
end

function YssTargets:PLAYER_REGEN_ENABLED()
	if self.db.profile.enemyOnly == nil then
		self.enemyOnly = nil
	end
	self:RAID_ROSTER_UPDATE()
end

function YssTargets:PLAYER_REGEN_DISABLED()
	if self.db.profile.enemyOnly == nil then
		self.enemyOnly = true
	end
	self:RAID_ROSTER_UPDATE()
end

function YssTargets:RAID_ROSTER_UPDATE()
	for unit in pairs(raidIDs) do
		if UnitExists(unit) then
			self:UNIT_TARGET(nil, unit)
		end
	end
end

local guid
function YssTargets:UNIT_TARGET(event, unit)
	if not self.db.profile.RaidInstanceOnly or select(2, IsInInstance()) == "raid" then
		if raidIDs[unit] and self.db.profile.enable then
			if Raid[unit] and GUIDs[Raid[unit]] then
				GUIDs[Raid[unit]][unit] = nil
				GUIDs[Raid[unit]].count = GUIDs[Raid[unit]].count - 1
				if GUIDs[Raid[unit]].count == 0 then
					GUIDs[Raid[unit]] = nil
				end
				Raid[unit] = nil
			end
			if self.enemyOnly and not UnitIsEnemy("player",unit..'target') then return end
			guid = UnitGUID(unit..'target')
			if guid then
				Raid[unit] = guid
				GUIDs[guid] = GUIDs[guid] or {['count'] = 0,}
				GUIDs[guid].count = GUIDs[guid].count + 1
				GUIDs[guid][unit] = true
			end
			if next(GUIDs) then
				if not self:GetScript('OnUpdate') then
					self:SetScript('OnUpdate', OnUpdate)
				end
			else
				self:SetScript('OnUpdate', nil)
			end
			updatetip = true
			self:UpdateBars()
		end
	elseif next(sortedbars) then
		self:ResetBars()
	end
end

local playerClass, englishClass
function YssTargets:UpdateToT(bar)
	local target = bar.unit..'target'
	if UnitName(target) then
		bar.target = bar.target or self:GetBar()
		bar.target.unit = target
		bar.target:SetPoint('LEFT', bar, 'RIGHT')
		bar.target.Ttext:SetText(UnitName(target))
		bar.target:UpdateValue(UnitHealth(target), UnitHealthMax(target))
		bar.target:SetThreat(target, bar.unit)
		bar.target:SetRaidIcon('RIGHT', GetRaidTargetIndex(target))
		
		local _, _, _, _, Tthreatvalue = UnitDetailedThreatSituation(target, bar.unit)
		local _, _, _, _, Pthreatvalue = UnitDetailedThreatSituation('player', bar.unit)
		if Tthreatvalue and Pthreatvalue and not UnitIsUnit("player", target) then
			if Pthreatvalue-Tthreatvalue > 0 then
				bar.target.Ltext:SetText(self:formatNumber((Pthreatvalue-Tthreatvalue)/100))
			else
				bar.target.Ltext:SetText('-'..self:formatNumber(mabs(Pthreatvalue-Tthreatvalue)/100))
			end
		else
			bar.target.Ltext:SetText('')
		end
	
		playerClass, englishClass = UnitClass(target)
		if englishClass then
			bar.target.Ttext:SetTextColor(RAID_CLASS_COLORS[englishClass].r, RAID_CLASS_COLORS[englishClass].g, RAID_CLASS_COLORS[englishClass].b)
			bar.target.englishClass = englishClass
		else
		bar.target.englishClass = nil
		end
	elseif bar.target then
		bar.target = self:RecycleBar(bar.target)
	end
end

local unit, count
function YssTargets:UpdateBars()
	wipe(sortedGUIDs)
	for guid in pairs(GUIDs) do
		sortedGUIDs[#sortedGUIDs+1] = guid
	end
	tsort(sortedGUIDs, sortf[self.db.profile.sort])
	count = 1
	for i, guid in ipairs(sortedGUIDs) do
		unit = next(GUIDs[guid], 'count')
		for unit in pairs(GUIDs[guid]) do
			if unit and guid == UnitGUID(unit..'target') and UnitName(unit..'target') and UnitName(unit..'target') ~= '' and UnitName(unit..'target') ~= UNKNOWN then
				unit = unit..'target'
				sortedbars[count] = sortedbars[count] or self:GetBar()
				sortedbars[count]:SetRaidIcon('LEFT', GetRaidTargetIndex(unit))
				sortedbars[count]:SetThreat('player', unit)
				sortedbars[count].guid = guid
				sortedbars[count].unit = unit
				sortedbars[count]:UpdateValue(UnitHealth(unit), UnitHealthMax(unit))
				sortedbars[count].Ttext:SetText(UnitName(unit))
				playerClass, englishClass = UnitClass(unit)
				if englishClass then
					sortedbars[count].Ttext:SetTextColor(RAID_CLASS_COLORS[englishClass].r, RAID_CLASS_COLORS[englishClass].g, RAID_CLASS_COLORS[englishClass].b)
					sortedbars[count].englishClass = englishClass
				else
					sortedbars[count].englishClass = nil
				end
				sortedbars[count].Ltext:SetText(GUIDs[guid].count)
				if count == 1 then
					sortedbars[count]:SetPoint(self.db.profile.anchor, self.anchor, anchors[self.db.profile.anchor])
				else
					sortedbars[count]:SetPoint(self.db.profile.anchor, sortedbars[count-1], anchors[self.db.profile.anchor])
				end
				if self.db.profile.ToT then
					self:UpdateToT(sortedbars[count])
				elseif sortedbars[count].target then
					sortedbars[count].target = self:RecycleBar(sortedbars[count].target)
				end
				count = count + 1
				break
			end
		end
	end
	for i=count, #sortedbars do
		sortedbars[i] = self:RecycleBar(sortedbars[i])
	end
	if currentbar then
		self:ShowTooltip(currentbar)
	end
end

local function GetAnchors(cell)
	local x, y = cell:GetCenter()
	local vertical
	if y < GetScreenHeight() / 2 then
		vertical = "BOTTOM"
	else
		vertical = "TOP"
	end
	if x < GetScreenWidth() / 2 then
		return vertical.."LEFT", vertical.."RIGHT"
	else
		
		return vertical.."RIGHT", vertical.."LEFT"
	end
end

local targetslist = {}
local targetslistdead = {}
local targetslistClass = {}
local targetslistName = {}

--|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_8:0|t
local function TTSort(a,b)
	return targetslistName[a] < targetslistName[b]
end

function YssTargets:UpdateList(bar)
	wipe(targetslist)
	wipe(targetslistdead)
	wipe(targetslistClass)
	wipe(targetslistName)
	for unit in pairs(GUIDs[bar.guid]) do
		if unit ~= 'count' then
			if UnitIsDeadOrGhost(unit) then
				targetslistdead[#targetslistdead+1] = unit
			else
				targetslist[#targetslist+1] = unit
			end
			targetslistName[unit] = UnitName(unit)
			local playerClass, englishClass = UnitClass(unit)
			targetslistClass[unit] = englishClass
		end
	end
	tsort(targetslist, TTSort)
	tsort(targetslistdead, TTSort)
end

function YssTargets:ShowTooltip() -- self is the bar frame and not addon table
	if not GUIDs[self.guid] then return end
	if updatetip then
		GameTooltip:ClearLines()
		GameTooltip:SetOwner(self, "ANCHOR_NONE")
		GameTooltip:ClearAllPoints()
		local a1, a2 = GetAnchors(self)
		GameTooltip:SetPoint(a1, self, a2)
		GameTooltip:AddLine('Targeting: '..YssTargets:ClassColor(self.Ttext:GetText(), self.englishClass))
		YssTargets:UpdateList(self)
		for i=1, #targetslist do
			GameTooltip:AddLine(targetslistName[targetslist[i]], RAID_CLASS_COLORS[targetslistClass[targetslist[i]]].r, RAID_CLASS_COLORS[targetslistClass[targetslist[i]]].g, RAID_CLASS_COLORS[targetslistClass[targetslist[i]]].b)
		end
		for i=1, #targetslistdead do
			GameTooltip:AddLine('|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_8:0|t '..targetslistName[targetslistdead[i]], RAID_CLASS_COLORS[targetslistClass[targetslistdead[i]]].r, RAID_CLASS_COLORS[targetslistClass[targetslistdead[i]]].g, RAID_CLASS_COLORS[targetslistClass[targetslistdead[i]]].b)
		end
	end
	--updatetip = false
	GameTooltip:Show()
end

local chattargets = {}
function YssTargets:ReportTargets() -- self is the bar frame and not addon table
	if not GUIDs[self.guid] or not IsModifierKeyDown() then return end
	wipe(chattargets)
	YssTargets:UpdateList(self)
	for i=1, #targetslist do
		chattargets[#chattargets+1] = targetslistName[targetslist[i]]
	end
	for i=1, #targetslistdead do
		chattargets[#chattargets+1] = '{rt8}'..targetslistName[targetslistdead[i]]
	end
	
	local stringtargets = 'People targeting '..self.Ttext:GetText()..': '..table.concat(chattargets, ", ")
	local start, maxlen, c = 255, 255, 4

	if string.len(stringtargets) < maxlen then
		SendChatMessage(stringtargets, "RAID")
	else
		while string.len(stringtargets) > 0 do
			local pos = string.find (stringtargets, ', ', start-c, true)
			if string.len(stringtargets) < maxlen then
				SendChatMessage(stringtargets, "RAID")
				stringtargets = ''
			elseif pos and pos < maxlen then
				SendChatMessage(string.sub(stringtargets, 1, pos-1), "RAID")
				stringtargets = string.sub(stringtargets, pos+2)
				start = maxlen
			else
				start = start-c
			end
		end
	end
end

function YssTargets:ClassColor(s, englishclass)
	if not englishclass then return s end 
	return string.format('|cff%02x%02x%02x%s|r', RAID_CLASS_COLORS[englishclass].r*255, RAID_CLASS_COLORS[englishclass].g*255, RAID_CLASS_COLORS[englishclass].b*255, s)
end

do
	local barRepo = {}
	
	local function UpdateValue(bar, curVal, maxVal)
		bar:SetMinMaxValues(0, maxVal)
		bar:SetValue(curVal)
		if maxVal == 0 then maxVal = 1 end
		if curVal/maxVal > .5 then
			bar:SetStatusBarColor(2*(1-(curVal/maxVal)),1,0,1)
		else
			bar:SetStatusBarColor(1,2*curVal/maxVal,0,1)
		end
		bar.Rtext:SetText(YssTargets:formatNumber(curVal)..'/'..YssTargets:formatNumber(maxVal)..' '..math.ceil(curVal/maxVal*100)..'%')
	end
	
	local RI
	local function SetRaidIcon(bar, side, raidTargetIconIndex)
		if side == 'LEFT' then
			RI = bar.Licon
		else
			RI = bar.Ricon
		end
		if raidTargetIconIndex then
			raidTargetIconIndex = raidTargetIconIndex - 1;
			local left, right, top, bottom;
			local coordIncrement = RAID_TARGET_ICON_DIMENSION / RAID_TARGET_TEXTURE_DIMENSION;
			left = mod(raidTargetIconIndex , RAID_TARGET_TEXTURE_COLUMNS) * coordIncrement;
			right = left + coordIncrement;
			top = floor(raidTargetIconIndex / RAID_TARGET_TEXTURE_ROWS) * coordIncrement;
			bottom = top + coordIncrement;
			RI.texture:SetTexCoord(left, right, top, bottom)
			RI:Show()
		else
			RI:Hide()
		end
	end
	
	local r,g,b
	local function SetThreat(bar, unit, mob)
		local _, status, _, rawthreatpct = UnitDetailedThreatSituation(unit, mob)
		if status == 3 or status == 2 then
			r, g, b = GetThreatStatusColor(status)
		elseif rawthreatpct and rawthreatpct > 90 then
			r, g, b = GetThreatStatusColor(1)
		else
			return bar.threat:Hide()
		end
		bar.threat:SetVertexColor(r,g,b)
		return bar.threat:Show()
	end
	
	local function leave(bar)
		currentbar = nil
		GameTooltip:Hide()
	end
	
	function YssTargets:GetBar()
		local bar
		if next(barRepo) then
			bar = barRepo[#barRepo]
			barRepo[#barRepo] = nil
		else
			bar = CreateFrame("StatusBar")
			bar:EnableMouse()
			bar:SetScript("OnEnter", YssTargets.ShowTooltip)
			bar:SetScript("OnMouseDown", YssTargets.ReportTargets)
			bar:SetScript("OnLeave", leave)
			local background = bar:CreateTexture(nil, "BACKGROUND")
				background:SetAllPoints(bar)
			bar.background = background
			local threat = bar:CreateTexture(nil, "BACKGROUND")
				threat:SetBlendMode("BLEND")
				threat:SetAlpha(self.db.profile.threattexturealpha)
				threat:SetDrawLayer("OVERLAY")
				threat:SetAllPoints(bar)
			bar.threat = threat
			local Ttext = bar:CreateFontString(nil, 'OVERLAY')
				Ttext:SetPoint("LEFT", bar)
				Ttext:SetPoint("TOP", bar,0,-1)
				Ttext:SetPoint("RIGHT", bar)
				Ttext:SetPoint("BOTTOM", bar, "CENTER")
				Ttext:SetJustifyH("CENTER")
				Ttext:SetJustifyV("TOP")
			bar.Ttext = Ttext
			local Rtext = bar:CreateFontString(nil, 'OVERLAY')
				Rtext:SetPoint("BOTTOMRIGHT", bar, -1, 1)
				Rtext:SetJustifyH("BOTTOM")
				Rtext:SetJustifyV("RIGHT")
			bar.Rtext = Rtext
			local Ltext = bar:CreateFontString(nil, 'OVERLAY')
				Ltext:SetPoint("BOTTOMLEFT", bar, 1, 1)
				Ltext:SetJustifyH("BOTTOM")
				Ltext:SetJustifyV("LEFT")
			bar.Ltext = Ltext
			bar:SetOrientation("HORIZONTAL")
			bar.UpdateValue = UpdateValue
			local Licon = CreateFrame("Frame")
				Licon:SetParent(bar)
				Licon:SetPoint("TOP")
				Licon:SetPoint("BOTTOM")
				Licon:SetPoint("RIGHT", bar, 'LEFT')
				local Ltexture = Licon:CreateTexture(nil, 'BACKGROUND')
					Ltexture:SetAllPoints(Licon)
					Ltexture:SetTexture([[Interface\TargetingFrame\UI-RaidTargetingIcons]])
				Licon.texture = Ltexture
				Licon:Hide()
			bar.Licon = Licon
			local Ricon = CreateFrame("Frame")
				Ricon:SetParent(bar)
				Ricon:SetPoint("TOP")
				Ricon:SetPoint("BOTTOM")
				Ricon:SetPoint("LEFT", bar, 'RIGHT')
				local Ltexture = Ricon:CreateTexture(nil, 'BACKGROUND')
					Ltexture:SetAllPoints(Ricon)
					Ltexture:SetTexture([[Interface\TargetingFrame\UI-RaidTargetingIcons]])
				Ricon.texture = Ltexture
				Ricon:Hide()
			bar.Ricon = Ricon
			bar.SetThreat = SetThreat
			bar.SetRaidIcon = SetRaidIcon
		end
		
		bar.Ttext:SetFont(Media:Fetch('font',self.db.profile.namefont),self.db.profile.namefontsize, "OUTLINE")
		bar.Rtext:SetFont(Media:Fetch('font',self.db.profile.healthfont),self.db.profile.healthfontsize)
		bar.Rtext:SetTextColor(self.db.profile.healthfontr,self.db.profile.healthfontg,self.db.profile.healthfontb,self.db.profile.healthfonta)
		bar.Ltext:SetFont(Media:Fetch('font',self.db.profile.countfont),self.db.profile.countfontsize)
		bar.Ltext:SetTextColor(self.db.profile.countfontr,self.db.profile.countfontb,self.db.profile.countfontg,self.db.profile.countfonta)
		bar:SetParent(UIParent)
		bar:SetStatusBarTexture(Media:Fetch('statusbar',self.db.profile.texture), 'ARTWORK')
		bar.background:SetTexture(Media:Fetch('statusbar',self.db.profile.texture))
		bar.threat:SetTexture(Media:Fetch('statusbar',self.db.profile.threattexture))
		bar.threat:SetAlpha(self.db.profile.threattexturealpha)
		bar.threat:Hide()
		bar:SetWidth(self.db.profile.width)
		bar.Licon:SetWidth(self.db.profile.height)
		bar.Ricon:SetWidth(self.db.profile.height)
		bar:SetHeight(self.db.profile.height)
		bar:Show()
		return bar
	end
	
	function YssTargets:RecycleBar(bar)
		if bar.target then
			bar.target = self:RecycleBar(bar.target)
		end
		bar.Licon:Hide()
		bar.Ricon:Hide()
		bar:Hide()
		bar.threat:Hide()
		bar:ClearAllPoints()
		barRepo[#barRepo+1] = bar
		--bar = nil
		return nil
	end
end

do
	local tb
	function YssTargets:ResetBars()
		if tb then self:RecycleBar(tb) end
		self.anchor:SetWidth(self.db.profile.width)
		self.anchor:SetHeight(self.db.profile.height)
		if self.db.profile.enemyOnly then
			self.enemyOnly = true
		else
			self.enemyOnly = nil
		end
		if YssTargets.TestBar then
			tb = self:GetBar()
			local health = math.random (5000, 50000000)	
			tb:UpdateValue(math.random (0, health), health)
			tb.Ttext:SetText('Bob')
			local englishClass = next(RAID_CLASS_COLORS)
			tb.Ttext:SetTextColor(RAID_CLASS_COLORS[englishClass].r, RAID_CLASS_COLORS[englishClass].g, RAID_CLASS_COLORS[englishClass].b)
			tb.Ltext:SetText(math.random (1, 40))
			tb:SetPoint(self.db.profile.anchor, self.anchor, anchors[self.db.profile.anchor])
			tb:SetRaidIcon('LEFT', math.random (1, 8))
			local threat = math.random (1,3)
			local r,g,b
			if threat == 3 then
				r,g,b = 1,0,0
			elseif threat == 2 then
				r,g,b = 1,.5,0
			else
				r,g,b = 1,1,0
			end
			--local r, g, b = GetThreatStatusColor(threat)
			tb.threat:SetVertexColor(r,g,b)
			tb.threat:Show()
		end
		for i=1, #sortedbars do
			sortedbars[i] = self:RecycleBar(sortedbars[i])
		end
		self:RAID_ROSTER_UPDATE()
	end
end

function YssTargets:Getter(info)
	return self.db.profile[info[#info]]
end

function YssTargets:Setter(info, val)
	self.db.profile[info[#info]] = val
	self:ResetBars()
end

function YssTargets:UpdateLDB()
	if self.db.profile.enable then
		self.obj.icon = "Interface\\AddOns\\YssTargets\\icon"
		self.obj.label = "|cFF33FF99YssTargets|r"
		self.obj.text  = "Enabled"
		self:RegisterEvent("UNIT_TARGET")
		self:RegisterEvent("RAID_ROSTER_UPDATE")
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		self:RegisterEvent("PLAYER_REGEN_DISABLED")
	else
		self.obj.icon = "Interface\\AddOns\\YssTargets\\icon_d"
		self.obj.label = "|cFF808080YssTargets|r"
		self.obj.text  = "Disabled"
		self:UnregisterEvent("UNIT_TARGET")
		self:UnregisterEvent("RAID_ROSTER_UPDATE")
		self:UnregisterEvent("PLAYER_REGEN_ENABLED")
		self:UnregisterEvent("PLAYER_REGEN_DISABLED")
	end
	self:ResetBars()
end

YssTargets.TestBar = false -- var to see if we show/hide our testbar
YssTargets.MainOption = {
	type="group",
	childGroups = 'select',
	handler = YssTargets,
	args = {
		enable = {
			type = "toggle",
			name = 'Enable',
			order = 1,
			get = 'Getter',
			set = function(info, val)
				YssTargets.db.profile.enable = val
				YssTargets:ResetBars()
				YssTargets:UpdateLDB()
			end,
		},
		RaidInstanceOnly = {
			type = "toggle",
			name = 'Raid Instance Only',
			desc = 'Bars will only show up in Raid Instances.',
			order = 4,
			get = 'Getter',
			set = 'Setter',
		},
		LibDBIcon = {
			type = "toggle",
			name = 'Minimap Icon',
			order = 5,
			get = function() return not YssTargets.db.profile.LibDBIcon.hide end,
			set = function(info, val)
				YssTargets.db.profile.LibDBIcon.hide = not val
				if val then
					icon:Show("YssTargets")
				else
					icon:Hide("YssTargets")
				end
			end,
		},
		TestBar = {
			type = "toggle",
			name = 'Show TestBar',
			order = 10,
			get = function() return YssTargets.TestBar end,
			set = function(info, val) 
				YssTargets.TestBar = val
				YssTargets:ResetBars()
			end,
		},
		ShowAnchor = {
			type = "toggle",
			name = 'Show Anchor',
			order = 20,
			get = function() return YssTargets.anchor:IsVisible() end,
			set = function(info, val) 
				if YssTargets.anchor:IsVisible() then
					YssTargets.anchor:Hide()
				else
					YssTargets.anchor:Show()
				end
			end,
		},
		enemyOnly = {
			type = "toggle",
			name = 'Enemys Only',
			desc = 'Tristate:|nChecked will display only enemys.|nGray will only show enemys during combat.|nUnchecked will show all targets.',
			tristate = true,
			order = 30,
			get = 'Getter',
			set = 'Setter',
		},
		ToT = {
			type = "toggle",
			name = 'Show Target of Target',
			order = 40,
			get = 'Getter',
			set = 'Setter',
		},
		bar = {
			name = 'Bar Options',
			type="group",
			inline = true,
			order = 50,
			args = {
				width = {
					name = 'Width',
					order = 30,
					type = "range",
					min = 10,
					max = 400,
					step = 1,
					get = 'Getter',
					set = 'Setter',
				},
				height = {
					name = 'Height',
					order = 40,
					type = "range",
					min = 10,
					max = 400,
					step = 1,
					get = 'Getter',
					set = 'Setter',
				},
				anchor = {
					type = "select",
					name = 'Bar Anchor',
					order = 50,
					values = {
						["TOP"] = 'TOP',
						["BOTTOM"] = 'BOTTOM',
					},
					get = 'Getter',
					set = 'Setter',
				},
				sort = {
					type = "select",
					name = 'Sort Direction',
					order = 60,
					values = {
						["normal"] = 'normal',
						["reverse"] = 'reverse',
					},
					get = 'Getter',
					set = 'Setter',
				},
				update = {
					name = 'Update Frequency',
					order = 70,
					type = "range",
					min = 0,
					max = 2,
					step = .01,
					get = 'Getter',
					set = 'Setter',
				},
				texture = {
					type = 'select',
					order = 80,
					dialogControl = 'LSM30_Statusbar', --Select your widget here
					name = 'Bar Texture',
					values = AceGUIWidgetLSMlists.statusbar, -- pull in your font list from LSM
					get = 'Getter',
					set = 'Setter',
				},
				threattexture = {
					type = 'select',
					order = 85,
					dialogControl = 'LSM30_Statusbar', --Select your widget here
					name = 'Threat Texture',
					values = AceGUIWidgetLSMlists.statusbar, -- pull in your font list from LSM
					get = 'Getter',
					set = 'Setter',
				},
				threattexturealpha = {
					type = 'range',
					order = 86,
					name = 'Threat Texture Alpha',
					min = 0,
					max = 1,
					step = .0001,
					isPercent = true,
					get = 'Getter',
					set = 'Setter',
				},
			},
		},
		name = {
			name = 'Name Options',
			type="group",
			inline = true,
			order = 90,
			args = {
				namefont = {
					type = 'select',
					order = 10,
					dialogControl = 'LSM30_Font', --Select your widget here
					name = 'Font',
					values = AceGUIWidgetLSMlists.font, -- pull in your font list from LSM
					get = 'Getter',
					set = 'Setter',
				},
				namefontsize = {
					name = 'Font Size',
					order = 20,
					type = "range",
					min = 1,
					max = 64,
					step = 1,
					get = 'Getter',
					set = 'Setter',
				},
			},
		},
		count = {
			name = 'Target Count Options',
			type="group",
			inline = true,
			order = 100,
			args = {
				countfont = {
					type = 'select',
					order = 10,
					dialogControl = 'LSM30_Font', --Select your widget here
					name = 'Font',
					values = AceGUIWidgetLSMlists.font, -- pull in your font list from LSM
					get = 'Getter',
					set = 'Setter',
				},
				countfontsize = {
					name = 'Font Size',
					order = 20,
					type = "range",
					min = 1,
					max = 26,
					step = 1,
					get = 'Getter',
					set = 'Setter',
				},
				countcolor = {
					name = 'Color',
					order = 30,
					type = "color",
					hasAlpha  = true,
					get = function() 
						return YssTargets.db.profile.countfontr,YssTargets.db.profile.countfontg,YssTargets.db.profile.countfontb,YssTargets.db.profile.countfonta
					end,
					set = function(info,r,g,b,a) 
						YssTargets.db.profile.countfontr,YssTargets.db.profile.countfontg,YssTargets.db.profile.countfontb,YssTargets.db.profile.countfonta = r,g,b,a
						YssTargets:ResetBars()
					end,
				},
			},
		},
		health = {
			name = 'Health Text Options',
			type="group",
			inline = true,
			order = 110,
			args = {
				healthfont = {
					type = 'select',
					order = 10,
					dialogControl = 'LSM30_Font', --Select your widget here
					name = 'Font',
					values = AceGUIWidgetLSMlists.font, -- pull in your font list from LSM
					get = 'Getter',
					set = 'Setter',
				},
				healthfontsize = {
					name = 'Font Size',
					order = 20,
					type = "range",
					min = 1,
					max = 26,
					step = 1,
					get = 'Getter',
					set = 'Setter',
				},
				healthcolor = {
					name = 'Color',
					order = 30,
					type = "color",
					hasAlpha  = true,
					get = function() 
						return YssTargets.db.profile.healthfontr,YssTargets.db.profile.healthfontg,YssTargets.db.profile.healthfontb,YssTargets.db.profile.healthfonta
					end,
					set = function(info,r,g,b,a) 
						YssTargets.db.profile.healthfontr,YssTargets.db.profile.healthfontg,YssTargets.db.profile.healthfontb,YssTargets.db.profile.healthfonta = r,g,b,a
						YssTargets:ResetBars()
					end,
				},
			},
		},
	},
}

YssTargets.optframe = {}
LibStub("AceConfig-3.0"):RegisterOptionsTable('YssTargetsConfig', YssTargets.MainOption)
YssTargets.optframe.YssTargets = LibStub("AceConfigDialog-3.0"):AddToBlizOptions('YssTargetsConfig', 'YssTargets')
-- function addapted from the RegisterModuleOptions function in Mapster
function YssTargets:RegisterModuleOptions(name, optionTbl, displayName)
	local cname = "YssTargetsConfig_"..name
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(cname, optionTbl)
	self.optframe[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(cname, displayName, "YssTargets")
end

-- /script YssTargets:Test()
function YssTargets:Test()

	self.testobj = LibStub("LibDataBroker-1.1"):NewDataObject("YssTargetsTest", {
		icon = "Interface\\AddOns\\YssTargets\\icon",
		label = "|cFF33FF99YssTargets|r",
		text  = GetAddOnMemoryUsage("YssTargets"),
	})
	local mem = GetAddOnMemoryUsage("YssTargets")
	local frame = CreateFrame('frame')
	local timer = 0
	frame:SetScript('OnUpdate', function(self, a1) 
		if timer > .5 then
			YssTargets.testobj.text = string.format("%.3fK %.3fKc", GetAddOnMemoryUsage("YssTargets"), GetAddOnMemoryUsage("YssTargets")-mem)
			mem = GetAddOnMemoryUsage("YssTargets")
			timer = 0
		else
			timer = timer + a1
		end
	end)
	
end
